@@ -6,7 +6,7 @@
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
  *
- *      http://www.apache.org/licenses/LICENSE-2.0
+ *      https://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -16,29 +16,29 @@
  */
 package org.apache.commons.codec.digest;
 
+import java.nio.charset.StandardCharsets;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
 import java.security.SecureRandom;
 import java.util.Arrays;
 import java.util.Random;
-import java.util.concurrent.ThreadLocalRandom;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
-import org.apache.commons.codec.Charsets;
-
 /**
  * SHA2-based Unix crypt implementation.
  * <p>
  * Based on the C implementation released into the Public Domain by Ulrich Drepper &lt;drepper@redhat.com&gt;
  * http://www.akkadia.org/drepper/SHA-crypt.txt
+ * </p>
  * <p>
  * Conversion to Kotlin and from there to Java in 2012 by Christian Hammers &lt;ch@lathspell.de&gt; and likewise put
  * into the Public Domain.
+ * </p>
  * <p>
  * This class is immutable and thread-safe.
+ * </p>
  *
- * @version $Id$
  * @since 1.7
  */
 public class Sha2Crypt {
@@ -47,7 +47,7 @@ public class Sha2Crypt {
     private static final int ROUNDS_DEFAULT = 5000;
 
     /** Maximum number of rounds. */
-    private static final int ROUNDS_MAX = 999999999;
+    private static final int ROUNDS_MAX = 999_999_999;
 
     /** Minimum number of rounds. */
     private static final int ROUNDS_MIN = 1000;
@@ -77,12 +77,11 @@ public class Sha2Crypt {
      * See {@link Crypt#crypt(String, String)} for details.
      * </p>
      * <p>
-     * A salt is generated for you using {@link ThreadLocalRandom}; for more secure salts consider using
-     * {@link SecureRandom} to generate your own salts and calling {@link #sha256Crypt(byte[], String)}.
+     * A salt is generated for you using {@link SecureRandom}.
      * </p>
      *
      * @param keyBytes
-     *            plaintext to hash
+     *            plaintext to hash. Each array element is set to {@code 0} before returning.
      * @return complete hash value
      * @throws IllegalArgumentException
      *             when a {@link java.security.NoSuchAlgorithmException} is caught.
@@ -97,11 +96,11 @@ public class Sha2Crypt {
      * See {@link Crypt#crypt(String, String)} for details.
      * </p>
      * @param keyBytes
-     *            plaintext to hash
+     *            plaintext to hash. Each array element is set to {@code 0} before returning.
      * @param salt
-     *            real salt value without prefix or "rounds=". The salt may be null, in which case a salt is generated for
-     *            you using {@link SecureRandom}. If one does not want to use {@link SecureRandom}, you can pass your
-     *            own {@link Random} in {@link #sha256Crypt(byte[], String, Random)}.
+     *            real salt value without prefix or "rounds=". The salt may be null, in which case a salt
+     *            is generated for you using {@link SecureRandom}. If one does not want to use {@link SecureRandom},
+     *            you can pass your own {@link Random} in {@link #sha256Crypt(byte[], String, Random)}.
      * @return complete hash value including salt
      * @throws IllegalArgumentException
      *             if the salt does not match the allowed pattern
@@ -121,19 +120,20 @@ public class Sha2Crypt {
      * See {@link Crypt#crypt(String, String)} for details.
      * </p>
      * @param keyBytes
-     *            plaintext to hash
+     *            plaintext to hash. Each array element is set to {@code 0} before returning.
      * @param salt
      *            real salt value without prefix or "rounds=".
      * @param random
-     *            the instance of {@link Random} to use for generating the salt. Consider using {@link SecureRandom}
-     *            or {@link ThreadLocalRandom}.
+     *            the instance of {@link Random} to use for generating the salt.
+     *            Consider using {@link SecureRandom} for more secure salts.
      * @return complete hash value including salt
      * @throws IllegalArgumentException
      *             if the salt does not match the allowed pattern
      * @throws IllegalArgumentException
      *             when a {@link java.security.NoSuchAlgorithmException} is caught.
+     * @since 1.12
      */
-    public static String sha256Crypt(final byte[] keyBytes, String salt, Random random) {
+    public static String sha256Crypt(final byte[] keyBytes, String salt, final Random random) {
         if (salt == null) {
             salt = SHA256_PREFIX + B64.getRandomSalt(8, random);
         }
@@ -145,11 +145,13 @@ public class Sha2Crypt {
      * <p>
      * This is a nearly line by line conversion of the original C function. The numbered comments are from the algorithm
      * description, the short C-style ones from the original C code and the ones with "Remark" from me.
+     * </p>
      * <p>
      * See {@link Crypt#crypt(String, String)} for details.
+     * </p>
      *
      * @param keyBytes
-     *            plaintext to hash
+     *            plaintext to hash. Each array element is set to {@code 0} before returning.
      * @param salt
      *            real salt value without prefix or "rounds="; may not be null
      * @param saltPrefix
@@ -160,7 +162,7 @@ public class Sha2Crypt {
      *            {@link MessageDigest} algorithm identifier string
      * @return complete hash value including prefix and salt
      * @throws IllegalArgumentException
-     *             if the given salt is <code>null</code> or does not match the allowed pattern
+     *             if the given salt is {@code null} or does not match the allowed pattern
      * @throws IllegalArgumentException
      *             when a {@link NoSuchAlgorithmException} is caught
      * @see MessageDigestAlgorithms
@@ -187,7 +189,7 @@ public class Sha2Crypt {
             roundsCustom = true;
         }
         final String saltString = m.group(4);
-        final byte[] saltBytes = saltString.getBytes(Charsets.UTF_8);
+        final byte[] saltBytes = saltString.getBytes(StandardCharsets.UTF_8);
         final int saltLen = saltBytes.length;
 
         // 1. start digest A
@@ -266,7 +268,7 @@ public class Sha2Crypt {
 
         // 11. For each bit of the binary representation of the length of the
         // password string up to and including the highest 1-digit, starting
-        // from to lowest bit position (numeric value 1):
+        // from to the lowest bit position (numeric value 1):
         //
         // a) for a 1-digit add digest B to digest A
         //
@@ -327,13 +329,13 @@ public class Sha2Crypt {
         /*
          * Create byte sequence P.
          */
-        final byte[] pBytes = new byte[keyLen];
+        final byte[] bytes = new byte[keyLen];
         int cp = 0;
         while (cp < keyLen - blocksize) {
-            System.arraycopy(tempResult, 0, pBytes, cp, blocksize);
+            System.arraycopy(tempResult, 0, bytes, cp, blocksize);
             cp += blocksize;
         }
-        System.arraycopy(tempResult, 0, pBytes, cp, keyLen - cp);
+        System.arraycopy(tempResult, 0, bytes, cp, keyLen - cp);
 
         // 17. start digest DS
         /*
@@ -341,7 +343,7 @@ public class Sha2Crypt {
          */
         altCtx = DigestUtils.getDigest(algorithm);
 
-        // 18. repeast the following 16+A[0] times, where A[0] represents the first
+        // 18. repeat the following 16+A[0] times, where A[0] represents the first
         // byte in digest A interpreted as an 8-bit unsigned value
         //
         // add the salt to digest DS
@@ -395,13 +397,13 @@ public class Sha2Crypt {
              */
             ctx = DigestUtils.getDigest(algorithm);
 
-            // b) for odd round numbers add the byte sequense P to digest C
+            // b) for odd round numbers add the byte sequence P to digest C
             // c) for even round numbers add digest A/C
             /*
              * Add key or last result.
              */
             if ((i & 1) != 0) {
-                ctx.update(pBytes, 0, keyLen);
+                ctx.update(bytes, 0, keyLen);
             } else {
                 ctx.update(altResult, 0, blocksize);
             }
@@ -419,7 +421,7 @@ public class Sha2Crypt {
              * Add key for numbers not divisible by 7.
              */
             if (i % 7 != 0) {
-                ctx.update(pBytes, 0, keyLen);
+                ctx.update(bytes, 0, keyLen);
             }
 
             // f) for odd round numbers add digest A/C
@@ -430,7 +432,7 @@ public class Sha2Crypt {
             if ((i & 1) != 0) {
                 ctx.update(altResult, 0, blocksize);
             } else {
-                ctx.update(pBytes, 0, keyLen);
+                ctx.update(bytes, 0, keyLen);
             }
 
             // h) finish digest C.
@@ -531,7 +533,7 @@ public class Sha2Crypt {
          */
         // Is there a better way to do this with the JVM?
         Arrays.fill(tempResult, (byte) 0);
-        Arrays.fill(pBytes, (byte) 0);
+        Arrays.fill(bytes, (byte) 0);
         Arrays.fill(sBytes, (byte) 0);
         ctx.reset();
         altCtx.reset();
@@ -547,12 +549,11 @@ public class Sha2Crypt {
      * See {@link Crypt#crypt(String, String)} for details.
      * </p>
      * <p>
-     * A salt is generated for you using {@link ThreadLocalRandom}; for more secure salts consider using
-     * {@link SecureRandom} to generate your own salts and calling {@link #sha512Crypt(byte[], String)}.
+     * A salt is generated for you using {@link SecureRandom}
      * </p>
      *
      * @param keyBytes
-     *            plaintext to hash
+     *            plaintext to hash. Each array element is set to {@code 0} before returning.
      * @return complete hash value
      * @throws IllegalArgumentException
      *             when a {@link java.security.NoSuchAlgorithmException} is caught.
@@ -567,7 +568,7 @@ public class Sha2Crypt {
      * See {@link Crypt#crypt(String, String)} for details.
      * </p>
      * @param keyBytes
-     *            plaintext to hash
+     *            plaintext to hash. Each array element is set to {@code 0} before returning.
      * @param salt
      *            real salt value without prefix or "rounds=". The salt may be null, in which case a salt is generated
      *            for you using {@link SecureRandom}; if you want to use a {@link Random} object other than
@@ -586,27 +587,25 @@ public class Sha2Crypt {
         return sha2Crypt(keyBytes, salt, SHA512_PREFIX, SHA512_BLOCKSIZE, MessageDigestAlgorithms.SHA_512);
     }
 
-
-
     /**
      * Generates a libc6 crypt() compatible "$6$" hash value.
      * <p>
      * See {@link Crypt#crypt(String, String)} for details.
      * </p>
      * @param keyBytes
-     *            plaintext to hash
+     *            plaintext to hash. Each array element is set to {@code 0} before returning.
      * @param salt
-     *            real salt value without prefix or "rounds=". The salt may be null, in which case a salt is generated for
-     *            you using {@link ThreadLocalRandom}; for more secure salts consider using {@link SecureRandom} to
-     *            generate your own salts.
+     *            real salt value without prefix or "rounds=". The salt may be null, in which case a salt
+     *            is generated for you using {@link SecureRandom}.
      * @param random
-     *            the instance of {@link Random} to use for generating the salt. Consider using {@link SecureRandom}
-     *            or {@link ThreadLocalRandom}.
+     *            the instance of {@link Random} to use for generating the salt.
+     *            Consider using {@link SecureRandom} for more secure salts.
      * @return complete hash value including salt
      * @throws IllegalArgumentException
      *             if the salt does not match the allowed pattern
      * @throws IllegalArgumentException
      *             when a {@link java.security.NoSuchAlgorithmException} is caught.
+     * @since 1.12
      */
     public static String sha512Crypt(final byte[] keyBytes, String salt, final Random random) {
         if (salt == null) {
@@ -614,4 +613,14 @@ public class Sha2Crypt {
         }
         return sha2Crypt(keyBytes, salt, SHA512_PREFIX, SHA512_BLOCKSIZE, MessageDigestAlgorithms.SHA_512);
     }
+
+    /**
+     * Consider private.
+     *
+     * @deprecated Will be private in the next major version.
+     */
+    @Deprecated
+    public Sha2Crypt() {
+        // empty
+    }
 }
